package enseirb.myinpulse.service;

import enseirb.myinpulse.model.*;
import enseirb.myinpulse.service.database.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.ArrayList;
import java.util.List;

@Service
public class SharedApiService {

    protected static final Logger logger = LogManager.getLogger();

    private final AdministratorService administratorService;
    private final UserService userService;
    private final ProjectService projectService;
    private final EntrepreneurService entrepreneurService;
    private final SectionCellService sectionCellService;
    private final AppointmentService appointmentService;

    @Autowired
    SharedApiService(
            AdministratorService administratorService,
            UserService userService,
            ProjectService projectService,
            EntrepreneurService entrepreneurService,
            SectionCellService sectionCellService,
            AppointmentService appointmentService) {
        this.administratorService = administratorService;
        this.userService = userService;
        this.projectService = projectService;
        this.entrepreneurService = entrepreneurService;
        this.sectionCellService = sectionCellService;
        this.appointmentService = appointmentService;
    }

    // TODO: test
    Boolean isAnAdmin(String mail) {
        try {
            long userId = this.userService.getUserByEmail(mail).getIdUser();
            Administrator a = this.administratorService.getAdministratorById(userId);
            return true;
        } catch (ResponseStatusException e) {
            logger.info(e);
            return false;
        }
    }

    // TODO: test
    Boolean isAllowedToCheckProject(String mail, long projectId) {
        if (isAnAdmin(mail)) {
            return true;
        }
        User user = this.userService.getUserByEmail(mail);
        Entrepreneur entrepreneur = this.entrepreneurService.getEntrepreneurById(user.getIdUser());
        Project project = this.projectService.getProjectById(projectId);
        return entrepreneur.getProjectParticipation() == project;
    }

    // TODO filter this with date
    public Iterable<SectionCell> getSectionCells(
            long projectId, long sectionId, String date, String mail) {

        if (!isAllowedToCheckProject(mail, projectId)) {
            logger.warn(
                    "User {} tried to check section cells of the project {} but is not allowed to.",
                    mail,
                    projectId);
            throw new ResponseStatusException(
                    HttpStatus.UNAUTHORIZED, "You're not allowed to check this project");
        }
        Project project = this.projectService.getProjectById(projectId);
        return this.sectionCellService.getSectionCellsByProject(project, sectionId);
    }

    // TODO: test
    public Iterable<Entrepreneur> getEntrepreneursByProjectId(long projectId, String mail) {
        if (!isAllowedToCheckProject(mail, projectId)) {
            logger.warn(
                    "User {} tried to check the member of the project {} but is not allowed to.",
                    mail,
                    projectId);
            throw new ResponseStatusException(
                    HttpStatus.UNAUTHORIZED, "You're not allowed to check this project");
        }
        Project project = this.projectService.getProjectById(projectId);
        return this.entrepreneurService.GetEntrepreneurByProject(project);
    }

    // TODO: test, protect via email
    public Administrator getAdminByProjectId(long projectId, String mail) {
        if (!isAllowedToCheckProject(mail, projectId)) {
            logger.warn(
                    "User {} tried to check the admin of the project {} but is not allowed to.",
                    mail,
                    projectId);
            throw new ResponseStatusException(
                    HttpStatus.UNAUTHORIZED, "You're not allowed to check this project");
        }
        Project project = this.projectService.getProjectById(projectId);
        return project.getAdministrator();
    }

    // TODO
    public Iterable<Appointment> getAppointmentsByProjectId(long projectId, String mail) {
        if (!isAllowedToCheckProject(mail, projectId)) {
            logger.warn(
                    "User {} tried to check the appointments related to the project {} but is not allowed to.",
                    mail,
                    projectId);
            throw new ResponseStatusException(
                    HttpStatus.UNAUTHORIZED, "You're not allowed to check this project");
        }
        logger.info(
                "User {} tried to check the appointments related to the project {}",
                mail,
                projectId);
        Iterable<SectionCell> sectionCells =
                this.sectionCellService.getSectionCellsByProject(
                        projectService.getProjectById(projectId),
                        2L); // sectionId useless in this function ?
        List<Appointment> appointments = new ArrayList<Appointment>();
        sectionCells.forEach(
                sectionCell -> {
                    appointments.addAll(
                            this.sectionCellService.getAppointmentsBySectionCellId(
                                    sectionCell.getIdSectionCell()));
                });
        return appointments;
    }

    // TODO
    public void getPDFReport(long appointmentId, String mail) {
        long projectId =
                this.appointmentService
                        .getAppointmentById(appointmentId)
                        .getAppointmentListSectionCell()
                        .getFirst()
                        .getProjectSectionCell()
                        .getIdProject();
        if (!isAllowedToCheckProject(mail, projectId)) {
            logger.warn(
                    "User {} tried to generate the PDF report {} related to the appointment {} but is not allowed to.",
                    mail,
                    this.appointmentService
                            .getAppointmentById(appointmentId)
                            .getAppointmentReport()
                            .getIdReport(),
                    appointmentId);
            throw new ResponseStatusException(
                    HttpStatus.UNAUTHORIZED, "You're not allowed to check this project");
        }
        /* return this.appointmentService
        .getAppointmentById(appointmentId)
        .getAppointmentReport().getReportContent(); */
        // generate pdf from this string, and format it to be decent looking
        throw new ResponseStatusException(HttpStatus.NOT_IMPLEMENTED, "Not implemented yet");
    }

    // TODO
    public void createAppointmentRequest(Appointment appointment, String mail) {
        throw new ResponseStatusException(HttpStatus.NOT_IMPLEMENTED, "Not implemented yet");
    }
}
